import {Component, EventEmitter, Input, Output} from '@angular/core';
import {Project} from "../../utils/result-preview/result-preview";
import {properties} from "../../../../environments/environment";

@Component({
  selector: 'fundedBy',
  template: `
    <div class="uk-margin-small-bottom uk-flex uk-flex-between">
      <span *ngIf="viewAll && !lessBtn" class="clickable uk-h6 uk-flex uk-flex-middle" (click)="viewLessClick()">
        <icon class="uk-margin-small-right" name="arrow_back" flex="true" ratio="1.2"></icon>
        {{title}}
      </span>
      <span *ngIf="!viewAll || lessBtn" class="uk-text-light-grey">{{title}}</span>
      <a *ngIf="viewAll && lessBtn" (click)="viewAll = !viewAll; lessBtn=false;">View less</a>
      <a *ngIf="fundedByProjects && fundedByProjects.length > threshold && !viewAll"
         (click)="viewAllClick();">View more</a>
    </div>
		<div>
			<span *ngFor="let item of fundedByProjects.slice(0, viewAll?fundedByProjects.length:threshold) let i=index">
						<span class="uk-text-emphasis">
							<a class="uk-link-text">
								<span
										*ngIf="item['funderShortname'] || item['funderName']">{{item['funderShortname'] ? item['funderShortname'] : item['funderName']}}</span>
								<span *ngIf="!item['funderShortname'] && !item['funderName']">[no funder available]</span>
								<span
										*ngIf="item['acronym'] || item['title']">| {{ item['acronym'] ? item['acronym'] : item['title']}}</span>
							</a>
						</span>
						<div class="default-dropdown uk-margin-remove-top uk-dropdown"
								 uk-dropdown="pos: bottom-left; mode:click">
              <div class="uk-padding-small">
                <span>Project</span>
                <div class="uk-margin-bottom">
                  <a *ngIf="item.id" class="uk-h6 uk-margin-remove portal-link"
                     [queryParams]="{projectId: item.id}" [routerLink]="url">
                    {{item['acronym'] ? item['acronym'] : item['title']}}
                  </a>
                  <span *ngIf="!item.id" class="uk-h6 uk-margin-remove">
                    {{item['acronym'] ? item['acronym'] : item['title']}}
                  </span>
                  <div *ngIf="item.acronym && item.title" class="uk-text-meta">
                    {{item.title}}
                  </div>
                </div>
                <ul class="uk-list uk-padding-remove-left uk-margin-bottom">
                  <li *ngIf="item.funderShortname || item.funderName">
                    <span class="uk-text-meta">Funder: </span>
                    {{item.funderName ? item.funderName : item.funderShortname}}
                    <span *ngIf="item.funderShortname && item.funderName">
                      ({{item.funderShortname}})
                    </span>
                  </li>
                  <li *ngIf="item.code">
                    <span class="uk-text-meta">Project Code: </span>{{item.code}}
                  </li>
                  <li *ngIf="item.funding">
                    <span class="uk-text-meta">Funding stream: </span>{{item.funding}}
                  </li>
                </ul>
                <div *ngIf="item.provenanceAction || item.validated" class="uk-text-meta">
                  <span *ngIf="item.validated">Validated by funder</span>
                  <span *ngIf="item.provenanceAction && item.validated"> | </span>
                  <span *ngIf="item.provenanceAction">{{item.provenanceAction}}</span>
                </div>
              </div>
						</div>
						<span *ngIf="i < (fundedByProjects.slice(0, viewAll?fundedByProjects.length:threshold).length - 1)">, </span>
					</span>
		</div>
  `
})

export class FundedByComponent {
  @Input() fundedByProjects: Project[];
  @Input() viewAll: boolean = false;
  @Output() viewAllClicked  = new EventEmitter();
  public lessBtn: boolean = false;
  public threshold: number = 4;
  public url = properties.searchLinkToProject.split('?')[0];
  public title: string = "Funded by";

  public viewAllClick() {
    if(this.fundedByProjects.length <= this.threshold*2) {
      this.viewAll = true;
      this.lessBtn = true;
    } else {
      this.viewAll = true;
      this.viewAllClicked.emit('fundedBy');
    }
  }

  public viewLessClick() {
    this.viewAll = false;
    this.viewAllClicked.emit("");
  }
}
