import {Component, Input} from '@angular/core';
import {EnvProperties} from "../../properties/env-properties";
import {ErrorCodes} from "../../properties/errorCodes";
import {OpenaireEntities} from "../../properties/searchFields";

@Component({
    selector: 'search-tab',
    template: `
      <a *ngIf="fetch.searchUtils.status == errorCodes.LOADING" class="uk-invisible" 
         [queryParams]="params" [routerLink]="searchLinkToAdvancedPage"></a>
      
      <ng-container *ngIf="fetch.searchUtils.status != errorCodes.LOADING">
        <div class="uk-flex uk-flex-between uk-flex-middle uk-margin-bottom">
          <div class="uk-text-meta uk-text-large uk-text-uppercase">
            <span *ngIf="!customTitle && resultType"><span *ngIf="resultType != 'organization' && resultType != 'dataprovider' && resultType != 'project'">Recent</span>
              {{getEntityName(resultType)}}
            </span>
            <span *ngIf="customTitle">{{customTitle}}</span>
          </div>
          <div *ngIf="searchLinkToAdvancedPage">
            <a class="el-content uk-button uk-button-text" [queryParams]="params" [routerLink]="searchLinkToAdvancedPage">
              View all
              <span *ngIf="fetch.searchUtils.totalResults <= searchNumber">in search page</span>
            </a>
          </div>
        </div>

        <ng-content></ng-content>
        
        <errorMessages *ngIf="resultType" [status]="[fetch.searchUtils.status]" [type]="getEntityName(resultType)"
                       tab_error_class=true></errorMessages>

      </ng-container>

      <ng-container *ngIf="fetch.searchUtils.status == errorCodes.DONE">
        <search-result [properties]="properties"
                       [results]="fetch.results"
                       [status]="fetch.searchUtils.status"
                       [type]="resultType"  [showImpactFactors]="showImpactFactors" [showEnermaps]="showEnermaps">
        </search-result>

<!--        <div *ngIf="searchLinkToAdvancedPage && fetch.results?.length > 0" class="uk-text-right">-->
<!--          <a class="el-content uk-button uk-button-text" [queryParams]="params" [routerLink]="searchLinkToAdvancedPage">-->
<!--            View all-->
<!--            <span *ngIf="fetch.searchUtils.totalResults <= searchNumber">in search page</span>-->
<!--          </a>-->
<!--        </div>-->
      </ng-container>
    `
})

export class SearchTabComponent {
    @Input() public fetch;
    @Input() public resultType: string;
    @Input() public params: any;
    @Input() public searchNumber: number = 5;
    @Input() public searchLinkToAdvancedPage: string;
    @Input() properties: EnvProperties;
    @Input() showImpactFactors;
    @Input() customTitle;
    @Input() showEnermaps: boolean;

    public errorCodes: ErrorCodes = new ErrorCodes();

    public getEntityName(entityType: string): string {
      if (entityType == "publication") {
        return OpenaireEntities.PUBLICATIONS;
      } else if (entityType == "dataset") {
        return OpenaireEntities.DATASETS;
      } else if (entityType == "software") {
        return OpenaireEntities.SOFTWARE;
      } else if (entityType == "other") {
        return OpenaireEntities.OTHER;
      } else if (entityType == "result") {
        return OpenaireEntities.RESULTS;
      } else if (entityType == "project") {
        return OpenaireEntities.PROJECTS;
      } else if (entityType == "dataprovider") {
        return OpenaireEntities.DATASOURCES;
      } else if (this.resultType) {
        return this.resultType;
      }
    }
}
